//------------------------------------
//rendering della scena--normale luce per pixel
float4x4 worldViewProj;
float4x4 world;
float4x4 worldInverseTranspose;
float4x4 viewInverse;

float4 lightDir;
float4 lightAmbient;
float shininess= 10.0;
float4 lightDiffuse;

//texture per l'oggetto
texture diffuseTexture;
sampler TextureSampler = sampler_state 
{
    texture = <diffuseTexture>;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};


//variabili per la variazione dell'esposizione
half3 defog={0.0f,0.0f,0.0f};
half gamma=0.6f;
half exposure=0.5f;
float BlurWidth = 3.0f;

//rendering per il bloom
float SceneIntensity= 0.8f;
float GlowIntensity = 0.8f;


//rendering con illuminazione a 16bit
struct vertexInput {
    float3 position				: POSITION;
    float3 normal				: NORMAL;
    float2 texCoordDiffuse		: TEXCOORD0;
};

struct vertexOutput {
    float4 hPosition		: POSITION;
    float2 texCoordDiffuse	: TEXCOORD0;
    float3 normal			: TEXCOORD1;
    float3 position			: TEXCOORD2;
};


//------------------------------------
vertexOutput VS_TransformAndTexture(vertexInput IN) 
{
    vertexOutput OUT;
    OUT.hPosition = mul( float4(IN.position.xyz , 1.0) , worldViewProj);
    OUT.texCoordDiffuse = IN.texCoordDiffuse;	
    OUT.position = mul(IN.position, world).xyz;
    OUT.normal = mul(IN.normal, worldInverseTranspose); 
    return OUT;
}





//-----------------------------------
float4 PS_Textured( vertexOutput IN): COLOR0
{
    
    float4 diffuseTexture = tex2D( TextureSampler, IN.texCoordDiffuse );
    
	//calculate our vectors N, E, L, and H
    float3 worldEyePos = viewInverse[3].xyz;
    float3 E = normalize(worldEyePos - IN.position); //eye vector
    float3 L = -normalize(IN.position -lightDir.xyz); //light vector
    float3 H = normalize(E + L); //half angle vector

    float d=distance(IN.position.xyz,lightDir);
    float atten=1/(d * d + d + 1);
	//calculate the diffuse and specular contributions
    float3 N=IN.normal;
    float  diff = max(0 , dot(N,L));
    float  spec = pow( max(0 , dot(N,H) ) , shininess );
    if( diff <= 0 )
    {
        spec = 0;
    }
    
    return (diff * diffuseTexture * lightDiffuse + spec)*10000 * atten+lightAmbient;
	
}




//rendering della correzione della gamma
struct vertexQuadOut {
    float4 position				: POSITION;
    float2 texCoord				: TEXCOORD0;
    float4 color				: COLOR0;
};


vertexQuadOut VS_Quad(float4 position:POSITION,float2 tex:TEXCOORD0){
	vertexQuadOut outP;
	outP.color=1;
	outP.position=position;
	outP.texCoord=tex;
	
	return outP;
}


half4 PS_HDR(float2 texCoord:TEXCOORD0):COLOR0{
    half3 c = tex2D(TextureSampler, texCoord);
	
    c = max(0, c - defog);
    
    c *= exposure;
    
    
    // gamma correction - could use texture lookups for this
    c = pow(c, gamma);
    return half4(c.rgb, 1.0);
}






//questi 3 pass sono eseguiti per sfumare e correggere la scena
float2 WindowSize = {1024, 1024};// : VIEWPORTPIXELSIZE;



texture SceneMap ;
sampler SceneSampler = sampler_state 
{
    texture = <SceneMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};



texture HBlurMap;
sampler HBlurSampler = sampler_state 
{
    texture = <HBlurMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture FinalBlurMap;

sampler FinalBlurSampler = sampler_state 
{
    texture = <FinalBlurMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};


struct VS_OUTPUT_BLUR
{
    float4 Position   : POSITION;
    float2 TexCoord[8]: TEXCOORD0;
};


// effettua un blur
VS_OUTPUT_BLUR VS_Blur(float4 Position : POSITION, float2 TexCoord:TEXCOORD0,
					   uniform int nsamples,
					   uniform float2 direction
					   )
{
    VS_OUTPUT_BLUR OUT = (VS_OUTPUT_BLUR)0;
    OUT.Position = Position;
    float2 texelSize = BlurWidth / WindowSize;
    float2 s = TexCoord - texelSize*(nsamples-1)*0.5*direction;
    for(int i=0; i<nsamples; i++) {
    	OUT.TexCoord[i] = s + texelSize*i*direction;
    }
    return OUT;
}



// blur filter weights
const half weights7[7] = {
	0.05,
	0.1,
	0.2,
	0.3,
	0.2,
	0.1,
	0.05,
};	

//effettua un blur
half4 PS_Blur7(VS_OUTPUT_BLUR IN) : COLOR
{
    half4 c = 0;
    for(int i=0; i<7; i++) {
    	c += tex2D(HBlurSampler, IN.TexCoord[i]) * weights7[i];
   	}
    return c;
} 


//effettua una proporzione tra la scena e la scena con blur
half4 PS_Comp(float2 tex:TEXCOORD0) : COLOR
{   
	
	half4 orig = tex2D(SceneSampler, tex);
	half4 blur = tex2D(HBlurSampler, tex);
	return SceneIntensity*orig + GlowIntensity*blur;
}  

////////////////////////////////////////////////////////////
/////////////////////////////////////// techniques /////////
////////////////////////////////////////////////////////////



//-----------------------------------
technique textured
{
    pass p0 
    {		
		VertexShader = compile vs_1_1 VS_TransformAndTexture();
		PixelShader  = compile ps_2_0 PS_Textured();
    }
    
    pass p1
    {
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		VertexShader = compile vs_2_0 VS_Blur(7, float2(1, 0));
		PixelShader  = compile ps_2_0 PS_Blur7( );
    }
    
    pass p2
    {
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		VertexShader = compile vs_2_0 VS_Blur(7, float2(0, 1));
		PixelShader  = compile ps_2_0 PS_Blur7();
    }
    
    pass p3
    {
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		VertexShader = compile vs_2_0 VS_Quad();
		PixelShader  = compile ps_2_0 PS_Comp();
    }
    
    
    pass p4
    {
		VertexShader= compile vs_1_1 VS_Quad();
		PixelShader=compile ps_2_0 PS_HDR();
    }
}

















